import os
from subprocess import check_output
from textwrap import dedent, indent

import nbformat
from nbconvert import RSTExporter

# generate the list of available notebooks
from xfel_calibrate import notebooks

rst_exporter = RSTExporter()
with open("available_notebooks.rst", "w") as f:
    f.write(dedent("""
            .. _available_notebooks:

            Available Notebooks
            ===================

            The following notebooks are currently integrated into the European XFEL
            Offline Calibration tool chain.


            """))

    for detector in sorted(notebooks.notebooks.keys()):
        values = notebooks.notebooks[detector]
        f.write("{}\n".format(detector))
        f.write("{}\n".format("-"*len(detector)))
        f.write("\n")

        for caltype in sorted(values.keys()):
            data = values[caltype]

            nbpath = os.path.abspath("{}/../../../{}".format(__file__, data["notebook"]))
            with open(nbpath, "r") as nf:
                nb = nbformat.read(nf, as_version=4)
                def first_markdown_cell(nb):
                    for cell in nb.cells:
                        if cell.cell_type == 'markdown':
                            return cell
                mdcell = first_markdown_cell(nb)
                nb.cells = [mdcell]  # we only want this single cell
                body, _ = rst_exporter.from_notebook_node(nb)
                adjusted = []
                # adjust titles
                for line in body.split("\n"):
                    if line.startswith("=="):
                        line = line.replace("=", "+")
                    if line.startswith("--"):
                        line = line.replace("-", "~")
                    adjusted.append(line)
                f.write("\n".join(adjusted))
                f.write("\n")

            f.write("To invoke this notebook and display help use:\n\n")
            f.write(".. code-block:: bash\n\n")
            f.write("    xfel-calibrate {} {} --help\n\n".format(detector, caltype))
            f.write("The full parameter list of this notebook (with defaults is): \n\n")
            f.write(".. code-block:: bash\n\n")
            nb_help = ["xfel-calibrate", detector, caltype, "--help"]
            output = check_output(nb_help).decode('utf8')
            f.write(indent(output.replace("DETECTOR", detector).replace("TYPE", caltype), " "*4))
            f.write("\n\n")